@@ -1,9 +1,10 @@
 /*
- * "$Id: scsi-irix.c,v 1.4 2004/05/13 15:13:42 mike Exp $"
+ * "$Id: scsi-irix.c 8173 2008-12-08 21:44:15Z mike $"
  *
  *   IRIX SCSI printer support for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2003-2004 by Easy Software Products, all rights reserved.
+ *   Copyright 2007 by Apple Inc.
+ *   Copyright 2003-2005 by Easy Software Products, all rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or
  *   without modification, are permitted provided that the
@@ -65,7 +66,8 @@
 void
 list_devices(void)
 {
-  puts("direct scsi \"Unknown\" \"SCSI Printer\"");
+  printf("direct scsi \"Unknown\" \"%s\"\n",
+         _cupsLangString(cupsLangDefault(), _("SCSI Printer")));
 }
 
 
@@ -95,34 +97,62 @@ print_device(const char *resource,	/* I - SCSI device */
 
   if (strncmp(resource, "/dev/scsi/", 10) != 0)
   {
-    fprintf(stderr, "ERROR: Bad SCSI device file \"%s\"!\n", resource);
-    return (1);
+    _cupsLangPrintf(stderr, _("ERROR: Bad SCSI device file \"%s\"!\n"),
+                    resource);
+    return (CUPS_BACKEND_STOP);
   }
 
  /*
   * Open the SCSI device file...
   */
 
+  fputs("STATE: +connecting-to-device\n", stderr);
+
   do
   {
     if ((scsi_fd = open(resource, O_RDWR | O_EXCL)) == -1)
     {
+      if (getenv("CLASS") != NULL)
+      {
+       /*
+        * If the CLASS environment variable is set, the job was submitted
+	* to a class and not to a specific queue.  In this case, we want
+	* to abort immediately so that the job can be requeued on the next
+	* available printer in the class.
+	*/
+
+        _cupsLangPuts(stderr,
+	              _("INFO: Unable to contact printer, queuing on next "
+			"printer in class...\n"));
+
+       /*
+        * Sleep 5 seconds to keep the job from requeuing too rapidly...
+	*/
+
+	sleep(5);
+
+        return (1);
+      }
+
       if (errno != EAGAIN && errno != EBUSY)
       {
-	fprintf(stderr, "ERROR: Unable to open SCSI device \"%s\" - %s\n",
-        	resource, strerror(errno));
-	return (1);
+	_cupsLangPrintf(stderr,
+	                _("ERROR: Unable to open device file \"%s\": %s\n"),
+			resource, strerror(errno));
+	return (CUPS_BACKEND_FAILED);
       }
       else
       {
-        fprintf(stderr, "INFO: SCSI device \"%s\" busy; retrying...\n",
-	        resource);
+        _cupsLangPuts(stderr,
+	              _("INFO: Printer busy; will retry in 30 seconds...\n"));
         sleep(30);
       }
     }
   }
   while (scsi_fd == -1);
 
+  fputs("STATE: -connecting-to-device\n", stderr);
+
  /*
   * Now that we are "connected" to the port, ignore SIGTERM so that we
   * can finish out any page data the driver sends (e.g. to eject the
@@ -176,8 +206,9 @@ print_device(const char *resource,	/* I - SCSI device */
 	if (ioctl(scsi_fd, DS_ENTER, &scsi_req) < 0 ||
             scsi_req.ds_status != 0)
         {
-	  fprintf(stderr, "WARNING: SCSI command timed out (%d); retrying...\n",
-	          scsi_req.ds_status);
+	  _cupsLangPrintf(stderr,
+			  _("WARNING: SCSI command timed out (%d); "
+			    "retrying...\n"), scsi_req.ds_status);
           sleep(try + 1);
 	}
 	else
@@ -185,10 +216,10 @@ print_device(const char *resource,	/* I - SCSI device */
 
       if (try >= 10)
       {
-	fprintf(stderr, "ERROR: Unable to send print data (%d)\n",
-	        scsi_req.ds_status);
+	_cupsLangPrintf(stderr, _("ERROR: Unable to send print data (%d)\n"),
+			scsi_req.ds_status);
         close(scsi_fd);
-	return (1);
+	return (CUPS_BACKEND_FAILED);
       }
     }
 
@@ -201,10 +232,10 @@ print_device(const char *resource,	/* I - SCSI device */
 
   close(fd);
 
-  return (0);
+  return (CUPS_BACKEND_OK);
 }
 
 
 /*
- * End of "$Id: scsi-irix.c,v 1.4 2004/05/13 15:13:42 mike Exp $".
+ * End of "$Id: scsi-irix.c 8173 2008-12-08 21:44:15Z mike $".
  */
