@@ -1,9 +1,9 @@
 /*
- * "$Id: usb.c 6911 2007-09-04 20:35:08Z mike $"
+ * "$Id: usb.c 7687 2008-06-24 01:28:36Z mike $"
  *
  *   USB port backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -30,14 +30,7 @@
 #  include <ApplicationServices/ApplicationServices.h>
 #endif /* __APPLE__ */
 
-#include <cups/backend.h>
-#include <cups/cups.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
-#include <cups/string.h>
-#include <cups/i18n.h>
-#include <signal.h>
+#include "backend-private.h"
 
 #ifdef WIN32
 #  include <io.h>
@@ -62,9 +55,11 @@ int	print_device(const char *uri, const char *hostname,
  * Include the vendor-specific USB implementation...
  */
 
-#ifdef __APPLE__
+#ifdef HAVE_USB_H
+#  include "usb-libusb.c"
+#elif defined(__APPLE__)
 #  include "usb-darwin.c"
-#elif defined(__linux) || defined(__sun) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
+#elif defined(__linux) || defined(__sun) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__DragonFly__) || defined(__FreeBSD_kernel__)
 #  include "usb-unix.c"
 #else
 /*
@@ -266,5 +261,5 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 
 
 /*
- * End of "$Id: usb.c 6911 2007-09-04 20:35:08Z mike $".
+ * End of "$Id: usb.c 7687 2008-06-24 01:28:36Z mike $".
  */
