@@ -1,32 +1,26 @@
 /*
- * "$Id: html.c 5549 2006-05-19 19:39:28Z mike $"
+ * "$Id: html.c 8278 2009-01-22 20:35:12Z mike $"
  *
  *   HTML support functions for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007-2009 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
- *   cgiEndHTML()      - End a HTML page.
- *   cgiFormEncode()   - Encode a string as a form variable...
- *   cgiStartHTML()    - Start a HTML page.
- *   cgi_null_passwd() - Return a NULL password for authentication.
+ *   cgiEndHTML()           - End a HTML page.
+ *   cgiEndMultipart()      - End the delivery of a multipart web page.
+ *   cgiFormEncode()        - Encode a string as a form variable.
+ *   cgiStartHTML()         - Start a HTML page.
+ *   cgiStartMultipart()    - Start a multipart delivery of a web page.
+ *   cgiSupportsMultipart() - Does the browser support multi-part documents?
+ *   cgi_null_passwd()      - Return a NULL password for authentication.
  */
 
 /*
@@ -36,6 +30,14 @@
 #include "cgi-private.h"
 
 
+/*
+ * Local globals...
+ */
+
+static const char	*cgi_multipart = NULL;
+					/* Multipart separator, if any */
+
+
 /*
  * Local functions...
  */
@@ -59,7 +61,22 @@ cgiEndHTML(void)
 
 
 /*
- * 'cgiFormEncode()' - Encode a string as a form variable...
+ * 'cgiEndMultipart()' - End the delivery of a multipart web page.
+ */
+
+void
+cgiEndMultipart(void)
+{
+  if (cgi_multipart)
+  {
+    printf("\n%s--\n", cgi_multipart);
+    fflush(stdout);
+  }
+}
+
+
+/*
+ * 'cgiFormEncode()' - Encode a string as a form variable.
  */
 
 char *					/* O - Destination string */
@@ -153,6 +170,9 @@ cgiStartHTML(const char *title)		/* I - Title of page */
   * Tell the client to expect UTF-8 encoded HTML...
   */
 
+  if (cgi_multipart)
+    puts(cgi_multipart);
+
   puts("Content-Type: text/html;charset=utf-8\n");
 
  /*
@@ -166,6 +186,56 @@ cgiStartHTML(const char *title)		/* I - Title of page */
 }
 
 
+/*
+ * 'cgiStartMultipart()' - Start a multipart delivery of a web page.
+ */
+
+void
+cgiStartMultipart(void)
+{
+  puts("MIME-Version: 1.0\n"
+       "Content-Type: multipart/x-mixed-replace; boundary=\"CUPS-MULTIPART\"\n");
+  fflush(stdout);
+
+  cgi_multipart = "--CUPS-MULTIPART";
+}
+
+
+/*
+ * 'cgiSupportsMultipart()' - Does the browser support multi-part documents?
+ */
+
+int					/* O - 1 if multi-part supported, 0 otherwise */
+cgiSupportsMultipart(void)
+{
+  const char	*user_agent;		/* User-Agent string */
+  static int	supports_multipart = -1;/* Cached value */
+
+
+  if (supports_multipart < 0)
+  {
+   /*
+    * CUPS STR #3049: Apparently some browsers don't support multi-part
+    * documents, which makes them useless for many web sites.  Rather than
+    * abandoning those users, we'll offer a degraded single-part mode...
+    *
+    * Currently we know that anything based on Gecko, MSIE, and Safari all
+    * work.  We'll add more as they are reported/tested.
+    */
+
+    if ((user_agent = getenv("HTTP_USER_AGENT")) != NULL &&
+        (strstr(user_agent, " Gecko/") != NULL ||
+	 strstr(user_agent, " MSIE ") != NULL ||
+	 strstr(user_agent, " Safari/") != NULL))
+      supports_multipart = 1;
+    else
+      supports_multipart = 0;
+  }
+
+  return (supports_multipart);
+}
+
+
 /*
  * 'cgi_null_passwd()' - Return a NULL password for authentication.
  */
@@ -183,5 +253,5 @@ cgi_null_passwd(const char *prompt)	/* I - Prompt string (unused) */
 
 
 /*
- * End of "$Id: html.c 5549 2006-05-19 19:39:28Z mike $".
+ * End of "$Id: html.c 8278 2009-01-22 20:35:12Z mike $".
  */
