@@ -1,5 +1,5 @@
 /*
- * "$Id: localize.c 8627 2009-05-13 21:39:17Z mike $"
+ * "$Id: localize.c 9233 2010-08-10 06:15:55Z mike $"
  *
  *   PPD localization routines for the Common UNIX Printing System (CUPS).
  *
@@ -389,7 +389,7 @@ ppdLocalizeIPPReason(
 
 	valptr += 5;
 
-        while (*valptr && !isspace(*valptr & 255) && bufptr < bufend)
+        while (*valptr && !_cups_isspace(*valptr) && bufptr < bufend)
 	{
 	  if (*valptr == '%' && isxdigit(valptr[1] & 255) &&
 	      isxdigit(valptr[2] & 255))
@@ -427,7 +427,7 @@ ppdLocalizeIPPReason(
         * Skip this URI...
 	*/
 
-        while (*valptr && !isspace(*valptr & 255))
+        while (*valptr && !_cups_isspace(*valptr))
           valptr++;
       }
 
@@ -435,7 +435,7 @@ ppdLocalizeIPPReason(
       * Skip whitespace...
       */
 
-      while (isspace(*valptr & 255))
+      while (_cups_isspace(*valptr))
 	valptr ++;
     }
 
@@ -463,7 +463,7 @@ ppdLocalizeIPPReason(
         * Copy URI...
 	*/
 
-        while (*valptr && !isspace(*valptr & 255) && bufptr < bufend)
+        while (*valptr && !_cups_isspace(*valptr) && bufptr < bufend)
 	  *bufptr++ = *valptr++;
 
 	*bufptr = '\0';
@@ -476,7 +476,7 @@ ppdLocalizeIPPReason(
         * Skip this URI...
 	*/
 
-	while (*valptr && !isspace(*valptr & 255))
+	while (*valptr && !_cups_isspace(*valptr))
 	  valptr++;
       }
 
@@ -484,7 +484,7 @@ ppdLocalizeIPPReason(
       * Skip whitespace...
       */
 
-      while (isspace(*valptr & 255))
+      while (_cups_isspace(*valptr))
 	valptr ++;
     }
 
@@ -598,7 +598,7 @@ _ppdGetLanguages(ppd_file_t *ppd)	/* I - PPD file */
     * Skip leading whitespace...
     */
 
-    while (isspace(*ptr & 255))
+    while (_cups_isspace(*ptr))
       ptr ++;
 
     if (!*ptr)
@@ -608,7 +608,7 @@ _ppdGetLanguages(ppd_file_t *ppd)	/* I - PPD file */
     * Find the end of this language name...
     */
 
-    for (start = ptr; *ptr && !isspace(*ptr & 255); ptr ++);
+    for (start = ptr; *ptr && !_cups_isspace(*ptr); ptr ++);
 
     if (*ptr)
       *ptr++ = '\0';
@@ -777,5 +777,5 @@ ppd_ll_CC(char *ll_CC,			/* O - Country-specific locale name */
 
 
 /*
- * End of "$Id: localize.c 8627 2009-05-13 21:39:17Z mike $".
+ * End of "$Id: localize.c 9233 2010-08-10 06:15:55Z mike $".
  */
