@@ -1,9 +1,9 @@
 /*
- * "$Id: pstops.c 7978 2008-09-23 23:44:43Z mike $"
+ * "$Id: pstops.c 9286 2010-08-31 18:36:08Z mike $"
  *
  *   PostScript filter for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 2007-2010 by Apple Inc.
  *   Copyright 1993-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -121,14 +121,22 @@ typedef struct				/**** Document information ****/
 		*title;			/* Job name */
   int		copies;			/* Number of copies */
   const char	*ap_input_slot,		/* AP_FIRSTPAGE_InputSlot value */
-		*ap_manual_feed;	/* AP_FIRSTPAGE_ManualFeed value */
+		*ap_manual_feed,	/* AP_FIRSTPAGE_ManualFeed value */
+		*ap_media_color,	/* AP_FIRSTPAGE_MediaColor value */
+		*ap_media_type,		/* AP_FIRSTPAGE_MediaType value */
+		*ap_page_region,	/* AP_FIRSTPAGE_PageRegion value */
+		*ap_page_size;		/* AP_FIRSTPAGE_PageSize value */
   float		brightness;		/* brightness value */
   int		collate,		/* Collate copies? */
 		emit_jcl,		/* Emit JCL commands? */
 		fitplot;		/* Fit pages to media */
   float		gamma;			/* gamma value */
   const char	*input_slot,		/* InputSlot value */
-		*manual_feed;		/* ManualFeed value */
+		*manual_feed,		/* ManualFeed value */
+		*media_color,		/* MediaColor value */
+		*media_type,		/* MediaType value */
+		*page_region,		/* PageRegion value */
+		*page_size;		/* PageSize value */
   int		mirror,			/* doc->mirror/mirror pages */
 		number_up,		/* Number of pages on each sheet */
 		number_up_layout,	/* doc->number_up_layout of N-up pages */
@@ -251,8 +259,9 @@ main(int  argc,				/* I - Number of command-line args */
 
   if (argc < 6 || argc > 7)
   {
-    fprintf(stderr, _("Usage: %s job-id user title copies options [file]\n"),
-            argv[0]);
+    _cupsLangPrintf(stderr,
+                    _("Usage: %s job-id user title copies options [file]\n"),
+                    argv[0]);
     return (1);
   }
 
@@ -287,8 +296,8 @@ main(int  argc,				/* I - Number of command-line args */
 
     if ((fp = cupsFileOpen(argv[6], "r")) == NULL)
     {
-      fprintf(stderr, _("ERROR: Unable to open file \"%s\" - %s\n"),
-              argv[6], strerror(errno));
+      _cupsLangPrintf(stderr, _("ERROR: Unable to open file \"%s\" - %s\n"),
+                      argv[6], strerror(errno));
       return (1);
     }
   }
@@ -299,7 +308,7 @@ main(int  argc,				/* I - Number of command-line args */
 
   if ((len = cupsFileGetLine(fp, line, sizeof(line))) == 0)
   {
-    fputs(_("ERROR: Empty print file!\n"), stderr);
+    _cupsLangPuts(stderr, _("ERROR: Empty print file!\n"));
     return (1);
   }
 
@@ -433,15 +442,17 @@ add_page(pstops_doc_t *doc,		/* I - Document information */
 
   if (!doc->pages)
   {
-    fprintf(stderr, _("EMERG: Unable to allocate memory for pages array: %s\n"),
-            strerror(errno));
+    _cupsLangPrintf(stderr,
+                    _("EMERG: Unable to allocate memory for pages array: %s\n"),
+                    strerror(errno));
     exit(1);
   }
 
   if ((pageinfo = calloc(1, sizeof(pstops_page_t))) == NULL)
   {
-    fprintf(stderr, _("EMERG: Unable to allocate memory for page info: %s\n"),
-            strerror(errno));
+    _cupsLangPrintf(stderr,
+                    _("EMERG: Unable to allocate memory for page info: %s\n"),
+                    strerror(errno));
     exit(1);
   }
 
@@ -553,13 +564,13 @@ copy_bytes(cups_file_t *fp,		/* I - File to read from */
 
   if (cupsFileSeek(fp, offset) < 0)
   {
-    fprintf(stderr,
+    _cupsLangPrintf(stderr,
 #ifdef HAVE_LONG_LONG
-            _("ERROR: Unable to seek to offset %lld in file - %s\n"),
+		    _("ERROR: Unable to seek to offset %lld in file - %s\n"),
 #else
-            _("ERROR: Unable to seek to offset %ld in file - %s\n"),
+		    _("ERROR: Unable to seek to offset %ld in file - %s\n"),
 #endif /* HAVE_LONG_LONG */
-            CUPS_LLCAST offset, strerror(errno));
+		    CUPS_LLCAST offset, strerror(errno));
     return;
   }
 
@@ -642,7 +653,7 @@ copy_comments(cups_file_t  *fp,		/* I - File to read from */
 
 
       if (saw_pages)
-        fputs(_("ERROR: Duplicate %%Pages: comment seen!\n"), stderr);
+        _cupsLangPuts(stderr, _("ERROR: Duplicate %%Pages: comment seen!\n"));
 
       saw_pages = 1;
 
@@ -689,7 +700,8 @@ copy_comments(cups_file_t  *fp,		/* I - File to read from */
     else if (!strncmp(line, "%%BoundingBox:", 14))
     {
       if (saw_bounding_box)
-        fputs(_("ERROR: Duplicate %%BoundingBox: comment seen!\n"), stderr);
+        _cupsLangPuts(stderr,
+	              _("ERROR: Duplicate %%BoundingBox: comment seen!\n"));
       else if (strstr(line + 14, "(atend)"))
       {
        /*
@@ -700,7 +712,7 @@ copy_comments(cups_file_t  *fp,		/* I - File to read from */
 	              doc->bounding_box + 1, doc->bounding_box + 2,
 		      doc->bounding_box + 3) != 4)
       {
-	fputs(_("ERROR: Bad %%BoundingBox: comment seen!\n"), stderr);
+	_cupsLangPuts(stderr, _("ERROR: Bad %%BoundingBox: comment seen!\n"));
 
 	doc->bounding_box[0] = (int)PageLeft;
 	doc->bounding_box[1] = (int)PageBottom;
@@ -752,10 +764,10 @@ copy_comments(cups_file_t  *fp,		/* I - File to read from */
   }
 
   if (!saw_bounding_box)
-    fputs(_("ERROR: No %%BoundingBox: comment in header!\n"), stderr);
+    _cupsLangPuts(stderr, _("ERROR: No %%BoundingBox: comment in header!\n"));
 
   if (!saw_pages)
-    fputs(_("ERROR: No %%Pages: comment in header!\n"), stderr);
+    _cupsLangPuts(stderr, _("ERROR: No %%Pages: comment in header!\n"));
 
   if (!saw_for)
     WriteTextComment("For", doc->user);
@@ -1088,8 +1100,10 @@ copy_non_dsc(cups_file_t  *fp,		/* I - File to read from */
   * that may not print correctly...
   */
 
-  fputs(_("WARNING: This document does not conform to the Adobe Document "
-          "Structuring Conventions and may not print correctly!\n"), stderr);
+  _cupsLangPuts(stderr,
+                _("WARNING: This document does not conform to the Adobe "
+		  "Document Structuring Conventions and may not print "
+		  "correctly!\n"));
 
  /*
   * Then write a standard DSC comment section...
@@ -1277,13 +1291,13 @@ copy_page(cups_file_t  *fp,		/* I - File to read from */
 
   if (!parse_text(line + 7, &ptr, label, sizeof(label)))
   {
-    fputs(_("ERROR: Bad %%Page: comment in file!\n"), stderr);
+    _cupsLangPuts(stderr, _("ERROR: Bad %%Page: comment in file!\n"));
     label[0] = '\0';
     number   = doc->page;
   }
   else if (strtol(ptr, &ptr, 10) == LONG_MAX || !isspace(*ptr & 255))
   {
-    fputs(_("ERROR: Bad %%Page: comment in file!\n"), stderr);
+    _cupsLangPuts(stderr, _("ERROR: Bad %%Page: comment in file!\n"));
     number = doc->page;
   }
 
@@ -1316,6 +1330,18 @@ copy_page(cups_file_t  *fp,		/* I - File to read from */
 					        doc->ap_manual_feed,
                                             pageinfo->num_options,
 					    &(pageinfo->options));
+      pageinfo->num_options = cupsAddOption("MediaColor", doc->ap_media_color,
+                                            pageinfo->num_options,
+					    &(pageinfo->options));
+      pageinfo->num_options = cupsAddOption("MediaType", doc->ap_media_type,
+                                            pageinfo->num_options,
+					    &(pageinfo->options));
+      pageinfo->num_options = cupsAddOption("PageRegion", doc->ap_page_region,
+                                            pageinfo->num_options,
+					    &(pageinfo->options));
+      pageinfo->num_options = cupsAddOption("PageSize", doc->ap_page_size,
+                                            pageinfo->num_options,
+					    &(pageinfo->options));
     }
     else if (doc->page == (Duplex + 2))
     {
@@ -1331,6 +1357,18 @@ copy_page(cups_file_t  *fp,		/* I - File to read from */
 					        doc->manual_feed,
                                             pageinfo->num_options,
 					    &(pageinfo->options));
+      pageinfo->num_options = cupsAddOption("MediaColor", doc->media_color,
+                                            pageinfo->num_options,
+					    &(pageinfo->options));
+      pageinfo->num_options = cupsAddOption("MediaType", doc->media_type,
+                                            pageinfo->num_options,
+					    &(pageinfo->options));
+      pageinfo->num_options = cupsAddOption("PageRegion", doc->page_region,
+                                            pageinfo->num_options,
+					    &(pageinfo->options));
+      pageinfo->num_options = cupsAddOption("PageSize", doc->page_size,
+                                            pageinfo->num_options,
+					    &(pageinfo->options));
     }
   }
 
@@ -1353,7 +1391,8 @@ copy_page(cups_file_t  *fp,		/* I - File to read from */
                  bounding_box + 1, bounding_box + 2,
 		 bounding_box + 3) != 4)
       {
-        fputs(_("ERROR: Bad %%PageBoundingBox: comment in file!\n"), stderr);
+        _cupsLangPuts(stderr,
+	              _("ERROR: Bad %%PageBoundingBox: comment in file!\n"));
         memcpy(bounding_box, doc->bounding_box,
 	       sizeof(bounding_box));
       }
@@ -1761,7 +1800,7 @@ copy_prolog(cups_file_t  *fp,		/* I - File to read from */
     if (!strncmp(line, "%%EndProlog", 11))
       linelen = cupsFileGetLine(fp, line, linesize);
     else
-      fputs(_("ERROR: Missing %%EndProlog!\n"), stderr);
+      _cupsLangPuts(stderr, _("ERROR: Missing %%EndProlog!\n"));
   }
 
   doc_puts(doc, "%%EndProlog\n");
@@ -1832,7 +1871,7 @@ copy_setup(cups_file_t  *fp,		/* I - File to read from */
     if (!strncmp(line, "%%EndSetup", 10))
       linelen = cupsFileGetLine(fp, line, linesize);
     else
-      fputs(_("ERROR: Missing %%EndSetup!\n"), stderr);
+      _cupsLangPuts(stderr, _("ERROR: Missing %%EndSetup!\n"));
   }
 
   if (num_options > 0)
@@ -1996,7 +2035,10 @@ do_setup(pstops_doc_t *doc,		/* I - Document information */
   */
 
   if (doc->number_up > 1)
+  {
+    doc_puts(doc, "userdict/CUPSsetpagedevice/setpagedevice load put\n");
     doc_puts(doc, "userdict/setpagedevice{pop}bind put\n");
+  }
 
  /*
   * Changes to the transfer function must be made AFTER any
@@ -2076,9 +2118,9 @@ doc_printf(pstops_doc_t *doc,		/* I - Document information */
 
   if (bytes > sizeof(buffer))
   {
-    fprintf(stderr,
-            _("ERROR: doc_printf overflow (%d bytes) detected, aborting!\n"),
-            (int)bytes);
+    _cupsLangPrintf(stderr,
+		    _("ERROR: doc_printf overflow (%d bytes) detected, "
+		      "aborting!\n"), (int)bytes);
     exit(1);
   }
 
@@ -2207,7 +2249,7 @@ include_feature(
 
   if (sscanf(line + 17, "%254s%254s", name, value) != 2)
   {
-    fputs(_("ERROR: Bad %%IncludeFeature: comment!\n"), stderr);
+    _cupsLangPuts(stderr, _("ERROR: Bad %%IncludeFeature: comment!\n"));
     return (num_options);
   }
 
@@ -2217,22 +2259,23 @@ include_feature(
 
   if ((option = ppdFindOption(ppd, name + 1)) == NULL)
   {
-    fprintf(stderr, _("WARNING: Unknown option \"%s\"!\n"), name + 1);
+    _cupsLangPrintf(stderr, _("WARNING: Unknown option \"%s\"!\n"), name + 1);
     return (num_options);
   }
 
   if (option->section == PPD_ORDER_EXIT ||
       option->section == PPD_ORDER_JCL)
   {
-    fprintf(stderr, _("WARNING: Option \"%s\" cannot be included via "
-                      "IncludeFeature!\n"), name + 1);
+    _cupsLangPrintf(stderr, _("WARNING: Option \"%s\" cannot be included via "
+                              "IncludeFeature!\n"), name + 1);
     return (num_options);
   }
 
   if (!ppdFindChoice(option, value))
   {
-    fprintf(stderr, _("WARNING: Unknown choice \"%s\" for option \"%s\"!\n"),
-            value, name + 1);
+    _cupsLangPrintf(stderr,
+                    _("WARNING: Unknown choice \"%s\" for option \"%s\"!\n"),
+                    value, name + 1);
     return (num_options);
   }
 
@@ -2350,6 +2393,7 @@ set_pstops_options(
   ppd_attr_t	*attr;			/* PPD attribute */
   ppd_option_t	*option;		/* PPD option */
   ppd_choice_t	*choice;		/* PPD choice */
+  const char	*content_type;		/* Original content type */
 
 
  /*
@@ -2377,18 +2421,46 @@ set_pstops_options(
   doc->new_bounding_box[3] = INT_MIN;
 
  /*
-  * AP_FIRSTPAGE_InputSlot
+  * See what the source content type is.  When printing PostScript content we
+  * want to do scaling and orientation, but otherwise we don't want to change
+  * anything...
   */
 
-  doc->ap_input_slot = cupsGetOption("AP_FIRSTPAGE_InputSlot", num_options,
-                                     options);
+  if ((content_type = getenv("CONTENT_TYPE")) == NULL)
+    content_type = "application/postscript";
+
+  if (!strcasecmp(content_type, "application/postscript"))
+    Orientation = 0;
 
  /*
-  * AP_FIRSTPAGE_ManualFeed
+  * AP_FIRSTPAGE_* and the corresponding non-first-page options.
   */
 
+  doc->ap_input_slot  = cupsGetOption("AP_FIRSTPAGE_InputSlot", num_options,
+                                      options);
   doc->ap_manual_feed = cupsGetOption("AP_FIRSTPAGE_ManualFeed", num_options,
                                       options);
+  doc->ap_media_color = cupsGetOption("AP_FIRSTPAGE_MediaColor", num_options,
+                                      options);
+  doc->ap_media_type  = cupsGetOption("AP_FIRSTPAGE_MediaType", num_options,
+                                      options);
+  doc->ap_page_region = cupsGetOption("AP_FIRSTPAGE_PageRegion", num_options,
+                                      options);
+  doc->ap_page_size   = cupsGetOption("AP_FIRSTPAGE_PageSize", num_options,
+                                      options);
+
+  if ((choice = ppdFindMarkedChoice(ppd, "InputSlot")) != NULL)
+    doc->input_slot = choice->choice;
+  if ((choice = ppdFindMarkedChoice(ppd, "ManualFeed")) != NULL)
+    doc->manual_feed = choice->choice;
+  if ((choice = ppdFindMarkedChoice(ppd, "MediaColor")) != NULL)
+    doc->media_color = choice->choice;
+  if ((choice = ppdFindMarkedChoice(ppd, "MediaType")) != NULL)
+    doc->media_type = choice->choice;
+  if ((choice = ppdFindMarkedChoice(ppd, "PageRegion")) != NULL)
+    doc->page_region = choice->choice;
+  if ((choice = ppdFindMarkedChoice(ppd, "PageSize")) != NULL)
+    doc->page_size = choice->choice;
 
  /*
   * brightness
@@ -2404,8 +2476,8 @@ set_pstops_options(
 
     if (intval < 10 || intval > 1000)
     {
-      fprintf(stderr, _("ERROR: Unsupported brightness value %s, using "
-                        "brightness=100!\n"), val);
+      _cupsLangPrintf(stderr, _("ERROR: Unsupported brightness value %s, using "
+                                "brightness=100!\n"), val);
       doc->brightness = 1.0f;
     }
     else
@@ -2449,13 +2521,24 @@ set_pstops_options(
     doc->emit_jcl = 1;
 
  /*
-  * fitplot
+  * fitplot/fit-to-page/ipp-attribute-fidelity
+  *
+  * (Only for original PostScript content)
   */
 
-  if ((val = cupsGetOption("fitplot", num_options, options)) != NULL &&
-      (!strcasecmp(val, "true") || !strcasecmp(val, "on") ||
-       !strcasecmp(val, "yes")))
-    doc->fitplot = 1;
+  if (!strcasecmp(content_type, "application/postscript"))
+  {
+    if ((val = cupsGetOption("fitplot", num_options, options)) != NULL &&
+	!strcasecmp(val, "true"))
+      doc->fitplot = 1;
+    else if ((val = cupsGetOption("fit-to-page", num_options, options)) != NULL &&
+	     !strcasecmp(val, "true"))
+      doc->fitplot = 1;
+    else if ((val = cupsGetOption("ipp-attribute-fidelity", num_options,
+                                  options)) != NULL &&
+	     !strcasecmp(val, "true"))
+      doc->fitplot = 1;
+  }
 
  /*
   * gamma
@@ -2471,8 +2554,8 @@ set_pstops_options(
 
     if (intval < 1 || intval > 10000)
     {
-      fprintf(stderr, _("ERROR: Unsupported gamma value %s, using "
-                        "gamma=1000!\n"), val);
+      _cupsLangPrintf(stderr, _("ERROR: Unsupported gamma value %s, using "
+                                "gamma=1000!\n"), val);
       doc->gamma = 1.0f;
     }
     else
@@ -2482,19 +2565,9 @@ set_pstops_options(
     doc->gamma = 1.0f;
 
  /*
-  * InputSlot
+  * mirror/MirrorPrint
   */
 
-  if ((choice = ppdFindMarkedChoice(ppd, "InputSlot")) != NULL)
-    doc->input_slot = choice->choice;
-
- /*
-  * ManualFeed
-  */
-
-  if ((choice = ppdFindMarkedChoice(ppd, "ManualFeed")) != NULL)
-    doc->manual_feed = choice->choice;
-
   if ((choice = ppdFindMarkedChoice(ppd, "MirrorPrint")) != NULL)
   {
     val = choice->choice;
@@ -2524,9 +2597,9 @@ set_pstops_options(
           doc->number_up = intval;
 	  break;
       default :
-          fprintf(stderr,
-	          _("ERROR: Unsupported number-up value %d, using "
-		    "number-up=1!\n"), intval);
+          _cupsLangPrintf(stderr,
+			  _("ERROR: Unsupported number-up value %d, using "
+			    "number-up=1!\n"), intval);
           doc->number_up = 1;
 	  break;
     }
@@ -2558,8 +2631,8 @@ set_pstops_options(
       doc->number_up_layout = PSTOPS_LAYOUT_BTRL;
     else
     {
-      fprintf(stderr, _("ERROR: Unsupported number-up-layout value %s, using "
-                        "number-up-layout=lrtb!\n"), val);
+      _cupsLangPrintf(stderr, _("ERROR: Unsupported number-up-layout value %s, "
+                                "using number-up-layout=lrtb!\n"), val);
       doc->number_up_layout = PSTOPS_LAYOUT_LRTB;
     }
   }
@@ -2608,8 +2681,8 @@ set_pstops_options(
       doc->page_border = PSTOPS_BORDERDOUBLE2;
     else
     {
-      fprintf(stderr, _("ERROR: Unsupported page-border value %s, using "
-                        "page-border=none!\n"), val);
+      _cupsLangPrintf(stderr, _("ERROR: Unsupported page-border value %s, "
+                                "using page-border=none!\n"), val);
       doc->page_border = PSTOPS_BORDERNONE;
     }
   }
@@ -2703,8 +2776,7 @@ set_pstops_options(
     if ((doc->temp = cupsTempFile2(doc->tempfile,
                                    sizeof(doc->tempfile))) == NULL)
     {
-      fprintf(stderr, _("ERROR: Unable to create temporary file: %s\n"),
-              strerror(errno));
+      _cupsLangPrintError(_("ERROR: Unable to create temporary file"));
       exit(1);
     }
   }
@@ -2816,9 +2888,6 @@ start_nup(pstops_doc_t *doc,		/* I - Document information */
   if (doc->number_up > 1)
     doc_puts(doc, "userdict/ESPsave save put\n");
 
-  if (doc->mirror)
-    doc_printf(doc, "%.1f 0.0 translate -1 1 scale\n", PageWidth);
-
   pos   = (number - 1) % doc->number_up;
   pagew = PageRight - PageLeft;
   pagel = PageTop - PageBottom;
@@ -2867,6 +2936,9 @@ start_nup(pstops_doc_t *doc,		/* I - Document information */
   else if (doc->number_up > 1 || doc->fitplot)
     doc_printf(doc, "%.1f %.1f translate\n", PageLeft, PageBottom);
 
+  if (doc->mirror)
+    doc_printf(doc, "%.1f 0.0 translate -1 1 scale\n", PageWidth);
+
   switch (doc->number_up)
   {
     default :
@@ -3427,6 +3499,15 @@ write_options(
   * Then send them out...
   */
 
+  if (doc->number_up > 1)
+  {
+   /*
+    * Temporarily restore setpagedevice so we can set the options...
+    */
+
+    doc_puts(doc, "userdict/setpagedevice/CUPSsetpagedevice load put\n");
+  }
+
   if (doc_setup)
   {
     doc_puts(doc, doc_setup);
@@ -3438,9 +3519,18 @@ write_options(
     doc_puts(doc, any_setup);
     free(any_setup);
   }
+
+  if (doc->number_up > 1)
+  {
+   /*
+    * Disable setpagedevice again...
+    */
+
+    doc_puts(doc, "userdict/setpagedevice{pop}bind put\n");
+  }
 }
 
 
 /*
- * End of "$Id: pstops.c 7978 2008-09-23 23:44:43Z mike $".
+ * End of "$Id: pstops.c 9286 2010-08-31 18:36:08Z mike $".
  */
