@@ -1,5 +1,5 @@
 /*
- * "$Id: rasterbench.c 7376 2008-03-19 21:07:45Z mike $"
+ * "$Id: rasterbench.c 8938 2009-12-18 23:52:01Z mike $"
  *
  *   Raster benchmark program for the Common UNIX Printing System (CUPS).
  *
@@ -26,6 +26,7 @@
  * Include necessary headers...
  */
 
+#include <config.h>
 #include <cups/raster.h>
 #include <stdlib.h>
 #include <sys/time.h>
@@ -275,26 +276,26 @@ write_test(int         fd,		/* I - File descriptor to write to */
   * text with some whitespace.
   */
 
-  srand(time(NULL));
+  CUPS_SRAND(time(NULL));
 
   memset(data, 0, sizeof(data));
 
   for (y = 0; y < 28; y ++)
   {
-    for (x = rand() & 127, count = (rand() & 15) + 1;
+    for (x = CUPS_RAND() & 127, count = (CUPS_RAND() & 15) + 1;
          x < sizeof(data[0]);
          x ++, count --)
     {
       if (count <= 0)
       {
-	x     += (rand() & 15) + 1;
-	count = (rand() & 15) + 1;
+	x     += (CUPS_RAND() & 15) + 1;
+	count = (CUPS_RAND() & 15) + 1;
 
         if (x >= sizeof(data[0]))
 	  break;
       }
 
-      data[y][x] = rand();
+      data[y][x] = CUPS_RAND();
     }
   }
 
@@ -350,5 +351,5 @@ write_test(int         fd,		/* I - File descriptor to write to */
 
 
 /*
- * End of "$Id: rasterbench.c 7376 2008-03-19 21:07:45Z mike $".
+ * End of "$Id: rasterbench.c 8938 2009-12-18 23:52:01Z mike $".
  */
