@@ -1,5 +1,5 @@
 /*
- * "$Id: listen.c 8258 2009-01-15 19:12:15Z mike $"
+ * "$Id: listen.c 9120 2010-04-23 18:56:34Z mike $"
  *
  *   Server listening routines for the Common UNIX Printing System (CUPS)
  *   scheduler.
@@ -182,6 +182,20 @@ cupsdStartListening(void)
 	cupsdLogMessage(CUPSD_LOG_ERROR,
 			"Unable to open listen socket for address %s:%d - %s.",
 			s, p, strerror(errno));
+
+#ifdef AF_INET6
+       /*
+        * IPv6 is often disabled while DNS returns IPv6 addresses...
+	*/
+
+	if (lis->address.addr.sa_family != AF_INET6 &&
+	    (FatalErrors & CUPSD_FATAL_LISTEN))
+	  cupsdEndProcess(getpid(), 0);
+#else
+	if (FatalErrors & CUPSD_FATAL_LISTEN)
+	  cupsdEndProcess(getpid(), 0);
+#endif /* AF_INET6 */
+
 	continue;
       }
 
@@ -236,7 +250,8 @@ cupsdStartListening(void)
 	unlink(lis->address.un.sun_path);
 
        /*
-	* Save the curent umask and set it to 0...
+	* Save the current umask and set it to 0 so that all users can access
+	* the domain socket...
 	*/
 
 	mask = umask(0);
@@ -266,6 +281,10 @@ cupsdStartListening(void)
 			s, p, strerror(errno));
 	close(lis->fd);
 	lis->fd = -1;
+
+	if (FatalErrors & CUPSD_FATAL_LISTEN)
+	  cupsdEndProcess(getpid(), 0);
+
 	continue;
       }
 
@@ -278,7 +297,14 @@ cupsdStartListening(void)
 	cupsdLogMessage(CUPSD_LOG_ERROR,
 			"Unable to listen for clients on address %s:%d - %s.",
 			s, p, strerror(errno));
-	exit(errno);
+
+	close(lis->fd);
+	lis->fd = -1;
+
+	if (FatalErrors & CUPSD_FATAL_LISTEN)
+	  cupsdEndProcess(getpid(), 0);
+
+        continue;
       }
     }
 
@@ -327,11 +353,8 @@ cupsdStartListening(void)
                     "No Listen or Port lines were found to allow access via "
 		    "localhost!");
 
-   /*
-    * Commit suicide...
-    */
-
-    cupsdEndProcess(getpid(), 0);
+    if (FatalErrors & (CUPSD_FATAL_CONFIG | CUPSD_FATAL_LISTEN))
+      cupsdEndProcess(getpid(), 0);
   }
 
  /*
@@ -416,5 +439,5 @@ cupsdStopListening(void)
 
 
 /*
- * End of "$Id: listen.c 8258 2009-01-15 19:12:15Z mike $".
+ * End of "$Id: listen.c 9120 2010-04-23 18:56:34Z mike $".
  */
