@@ -1,9 +1,9 @@
 /*
- * "$Id: config.h 8771 2009-08-20 18:14:49Z mike $"
+ * "$Id: config.h 9233 2010-08-10 06:15:55Z mike $"
  *
- *   Configuration file for the Common UNIX Printing System (CUPS).
+ *   Configuration file for CUPS.
  *
- *   Copyright 2007-2009 by Apple Inc.
+ *   Copyright 2007-2010 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -24,7 +24,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
-#include <ctype.h>
 #include <io.h>
 
 
@@ -51,6 +50,27 @@
 #define write		_write
 
 
+/*
+ * Map the POSIX sleep() and usleep() functions to the Win32 Sleep() function...
+ */
+
+#define sleep(X)	Sleep(1000 * (X))
+#define usleep(X)	Sleep((X)/1000)
+
+
+/*
+ * Map various parameters to Posix style system calls
+ */
+
+#  define F_OK		00
+#  define W_OK		02
+#  define R_OK		04
+#  define O_RDONLY	_O_RDONLY
+#  define O_WRONLY	_O_WRONLY
+#  define O_CREATE	_O_CREAT
+#  define O_TRUNC	_O_TRUNC
+
+
 /*
  * Compiler stuff...
  */
@@ -63,18 +83,18 @@
  * Version of software...
  */
 
-#define CUPS_SVERSION "CUPS v1.4.0"
-#define CUPS_MINIMAL "CUPS/1.4.0"
+#define CUPS_SVERSION "CUPS v1.4.4"
+#define CUPS_MINIMAL "CUPS/1.4.4"
 
 
 /*
  * Default user and groups...
  */
 
-#define CUPS_DEFAULT_USER	"lp"
-#define CUPS_DEFAULT_GROUP	"sys"
-#define CUPS_DEFAULT_SYSTEM_GROUPS "admin"
-#define CUPS_DEFAULT_PRINTOPERATOR_AUTH "@admin @lpadmin"
+#define CUPS_DEFAULT_USER	""
+#define CUPS_DEFAULT_GROUP	""
+#define CUPS_DEFAULT_SYSTEM_GROUPS ""
+#define CUPS_DEFAULT_PRINTOPERATOR_AUTH ""
 
 
 /*
@@ -207,6 +227,13 @@
 /* #undef HAVE_CRYPT_H */
 
 
+/*
+ * Do we have <scsi/sg.h>?
+ */
+
+/* #undef HAVE_SCSI_SG_H */
+
+
 /*
  * Use <string.h>, <strings.h>, and/or <bstring.h>?
  */
@@ -246,9 +273,9 @@
  * Do we have the strXXX() functions?
  */
 
-#define HAVE_STRDUP
-#define HAVE_STRCASECMP
-#define HAVE_STRNCASECMP
+#define HAVE_STRDUP 1
+#define HAVE_STRCASECMP 1
+#define HAVE_STRNCASECMP 1
 /* #undef HAVE_STRLCAT */
 /* #undef HAVE_STRLCPY */
 
@@ -471,7 +498,6 @@
 
 /* #undef HAVE_LAUNCH_H */
 /* #undef HAVE_LAUNCHD */
-#define CUPS_DEFAULT_LAUNCHD_CONF ""
 
 
 /*
@@ -521,6 +547,13 @@
 /* #undef HAVE_CFBUNDLEPRIV_H */
 
 
+/*
+ * Do we have the SCDynamicStoreCopyComputerName function?
+ */
+
+/* #undef HAVE_SCDYNAMICSTORECOPYCOMPUTERNAME */
+
+
 /*
  * Do we have MacOSX 10.4's mbr_XXX functions()?
  */
@@ -574,7 +607,7 @@
  * Default GSS service name...
  */
 
-#define CUPS_DEFAULT_GSSSERVICENAME "ipp"
+#define CUPS_DEFAULT_GSSSERVICENAME "host"
 
 
 /*
@@ -625,10 +658,24 @@
  * Which random number generator function to use...
  */
 
+/* #undef HAVE_ARC4RANDOM */
 /* #undef HAVE_RANDOM */
-/* #undef HAVE_MRAND48 */
 /* #undef HAVE_LRAND48 */
 
+#ifdef HAVE_ARC4RANDOM
+#  define CUPS_RAND() arc4random()
+#  define CUPS_SRAND(v) arc4random_stir()
+#elif defined(HAVE_RANDOM)
+#  define CUPS_RAND() random()
+#  define CUPS_SRAND(v) srandom(v)
+#elif defined(HAVE_LRAND48)
+#  define CUPS_RAND() lrand48()
+#  define CUPS_SRAND(v) srand48(v)
+#else
+#  define CUPS_RAND() rand()
+#  define CUPS_SRAND(v) srand(v)
+#endif /* HAVE_ARC4RANDOM */
+
 
 /*
  * Do we have vproc_transaction_begin/end?
@@ -654,5 +701,5 @@
 #endif /* !_CUPS_CONFIG_H_ */
 
 /*
- * End of "$Id: config.h 8771 2009-08-20 18:14:49Z mike $".
+ * End of "$Id: config.h 9233 2010-08-10 06:15:55Z mike $".
  */
